package com.mindoo.dde.imageproperties;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.StringReader;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import lotus.domino.Document;
import lotus.domino.DxlExporter;
import lotus.domino.NotesException;
import lotus.domino.Session;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.util.Assert;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.progress.UIJob;
import org.eclipse.ui.views.properties.tabbed.AbstractPropertySection;
import org.eclipse.ui.views.properties.tabbed.ITabbedPropertyConstants;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import com.ibm.designer.domino.ide.resources.extensions.DesignerDesignElement;
import com.ibm.designer.domino.ide.resources.extensions.DesignerException;
import com.ibm.designer.domino.ui.commons.extensions.DesignerResource;
import com.ibm.notes.java.api.util.NotesSessionJob;
import com.mindoo.dde.imageproperties.internal.Activator;
import com.mindoo.dde.imageproperties.tools.Base64;
import com.mindoo.dde.imageproperties.tools.NodeTextExtractor;

public class WebImageSizeSection extends AbstractPropertySection {
	TabbedPropertySheetPage m_aTabbedPropertySheetPage;
	private Composite m_parent;

	private Text m_widthTextField;
	private CLabel m_widthLabel;
	private Text m_heightTextField;
	private CLabel m_heightLabel;
	private Group m_group;

	private boolean m_refreshNeeded;

	public WebImageSizeSection() {
		super();
	}

	@Override
	public void createControls(Composite parent, TabbedPropertySheetPage aTabbedPropertySheetPage) {
		super.createControls(parent, aTabbedPropertySheetPage);
		m_parent = parent;
		m_aTabbedPropertySheetPage = aTabbedPropertySheetPage;

		Composite composite = getWidgetFactory().createFlatFormComposite(parent);
		FormData data;

		m_group = getWidgetFactory().createGroup(composite, "Image size in px"); //$NON-NLS-1$
		m_group.setLayout(new FormLayout());


		data = new FormData();
		data.left = new FormAttachment(0, 0);
		data.top = new FormAttachment(0, ITabbedPropertyConstants.VSPACE);
		data.width=293;
		m_group.setLayoutData(data);

		Composite contentPane = getWidgetFactory().createFlatFormComposite(m_group);
		data = new FormData();
		data.left = new FormAttachment(0,0);
		data.right = new FormAttachment(100,0);
		data.top = new FormAttachment(0,0);
		contentPane.setLayoutData(data);

		m_widthTextField = getWidgetFactory().createText(contentPane, ""); //$NON-NLS-1$

		m_widthLabel = getWidgetFactory().createCLabel(contentPane, "Width:"); //$NON-NLS-1$
		data = new FormData();
		data.left = new FormAttachment(0, ITabbedPropertyConstants.HSPACE);
		data.top = new FormAttachment(m_widthTextField, 0, SWT.CENTER);
		data.width=60;
		m_widthLabel.setLayoutData(data);

		data = new FormData();
		data.left = new FormAttachment(m_widthLabel, 0);
		data.width = 40;
		data.top = new FormAttachment(0, ITabbedPropertyConstants.VSPACE);
		m_widthTextField.setLayoutData(data);

		m_heightTextField = getWidgetFactory().createText(contentPane, ""); //$NON-NLS-1$

		m_heightLabel = getWidgetFactory().createCLabel(contentPane, "Height:"); //$NON-NLS-1$
		data = new FormData();
		data.left = new FormAttachment(0, ITabbedPropertyConstants.HSPACE);
		data.top = new FormAttachment(m_heightTextField, 0, SWT.CENTER);
		data.width=60;
		m_heightLabel.setLayoutData(data);

		data = new FormData();
		data.left = new FormAttachment(m_heightLabel, 0);
		data.width = 40;
		data.top = new FormAttachment(m_widthTextField, ITabbedPropertyConstants.VSPACE);
		m_heightTextField.setLayoutData(data);
	}

	@Override
	public void dispose() {
		super.dispose();
	}

	@Override
	public void setInput(IWorkbenchPart part, ISelection selection) {
		if (isSelectionUnchanged(selection))
			return;

		Assert.isTrue(selection instanceof IStructuredSelection);
		super.setInput(part, selection);

		m_refreshNeeded=true;
	}

	public void refresh() {
		if (!m_refreshNeeded)
			return;

		m_refreshNeeded=false;

		ISelection selection=getSelection();
		if (selection instanceof IStructuredSelection) {
			Object input = ((IStructuredSelection) selection).getFirstElement();
			IResource resource = null;
			if (input instanceof IAdaptable) {
				resource=(IResource) ((IAdaptable)input).getAdapter(IResource.class);
			}
			else if (input instanceof IResource) {
				resource=(IResource)input;
			}
			if (resource!=null) {
				DesignerDesignElement de=DesignerResource.getDesignElement(resource);
				if (de!=null) {
					if (DesignerResource.TYPE_IMAGE.equals(de.getDesignElementType())) {
						FetchImageSize fetchJob=new FetchImageSize(de);
						fetchJob.schedule();
					}
				}
			}
		}
	}

	protected boolean isSelectionUnchanged(ISelection sel) {
		if ((getSelection() != null) && (getSelection().equals(sel))) {
			return true;
		}
		else
			return false;
	}

	@Override
	public void aboutToBeHidden() {
		super.aboutToBeHidden();
	}

	@Override
	public void aboutToBeShown() {
		super.aboutToBeShown();
	}

	protected class FetchImageSize extends NotesSessionJob {
		private DesignerDesignElement m_de;

		public FetchImageSize(DesignerDesignElement de) {
			super("Fetching image size");
			m_de=de;
		}

		protected Rectangle parseImageSize(org.w3c.dom.Document doc) throws UnsupportedImageFormatException {
			NodeList gifTag=doc.getElementsByTagName("gif");
			if (gifTag!=null && gifTag.getLength()>0) {
				Node firstNode=gifTag.item(0);
				if (firstNode instanceof Element) {
					String base64Code=NodeTextExtractor.getTextContent((Element)firstNode);
					byte[] imageData=Base64.decode(base64Code);
					Image image = new Image(Display.getCurrent(), new ByteArrayInputStream(imageData));
					if (image!=null) {
						Rectangle bounds=image.getBounds();
						image.dispose();
						if (bounds!=null) {
							return bounds;
						}
					}
					throw new UnsupportedImageFormatException("The platform could not load the image from the DXL data");
				}
				else
					throw new UnsupportedImageFormatException("Could not find the image data in the dxl code");
			}
			else {
				throw new UnsupportedImageFormatException("No supported image format could be found");
			}
		}

		protected void updateImageSize(final Rectangle newSize) {
			UIJob refreshJob=new UIJob("Refreshing image size fields") {

				@Override
				public IStatus runInUIThread(IProgressMonitor arg0) {
					int width=newSize.width;
					int height=newSize.height;
					if (m_widthTextField!=null)
						m_widthTextField.setText(Integer.toString(width));
					if (m_heightTextField!=null)
						m_heightTextField.setText(Integer.toString(height));
					return Status.OK_STATUS;
				}
			};
			refreshJob.schedule();
		}

		@Override
		protected IStatus runInNotesThread(Session session, IProgressMonitor progessMon) throws NotesException {
			String imageName;
			String notesUrl;
			try {
				imageName=m_de.getName();
				notesUrl=m_de.getNotesURL();
			} catch (DesignerException e) {
				return new Status(IStatus.ERROR,Activator.PLUGIN_ID,"Could not access the Notes url of the image design element",e);
			}
			DxlExporter dxlExporter=session.createDxlExporter();
			dxlExporter.setConvertNotesBitmapsToGIF(true);
			progessMon.worked(10);
			Object deObj=session.resolve(notesUrl);
			progessMon.worked(70);
			if (deObj instanceof Document) {
				Document deDoc=(Document) deObj;
				String dxlCode=dxlExporter.exportDxl(deDoc);

				DocumentBuilderFactory docBuilderFactory=DocumentBuilderFactory.newInstance();
				docBuilderFactory.setValidating(false);
				try {
					docBuilderFactory.setFeature("http://apache.org/xml/features/nonvalidating/load-external-dtd", false);
				} catch (ParserConfigurationException e1) {
					e1.printStackTrace();
				} //Disable DTD validation
			      
				DocumentBuilder docBuilder;
				try {
					docBuilder=docBuilderFactory.newDocumentBuilder();
				} catch (ParserConfigurationException e) {
					return new Status(IStatus.ERROR,Activator.PLUGIN_ID,"Could not get the system's xml parser",e);
				}
				try {
					org.w3c.dom.Document xmlDoc=docBuilder.parse(new InputSource(new StringReader(dxlCode)));
					Rectangle imgSize=parseImageSize(xmlDoc);
					updateImageSize(imgSize);
					
					return Status.OK_STATUS;
				} catch (SAXException e) {
					return new Status(IStatus.ERROR,Activator.PLUGIN_ID,"Could not parse the image's DXL code",e);
				} catch (IOException e) {
					return new Status(IStatus.ERROR,Activator.PLUGIN_ID,"Could not parse the image's DXL code",e);
				} catch (UnsupportedImageFormatException e) {
					return new Status(IStatus.WARNING,Activator.PLUGIN_ID,"Could not retrieve the image size",e);
				}

			}
			return new Status(IStatus.WARNING,Activator.PLUGIN_ID,"Could not retrieve the image size for image "+imageName);
		}
	}

	protected static class UnsupportedImageFormatException extends Exception {
		public UnsupportedImageFormatException() {
			super();
		}

		public UnsupportedImageFormatException(String paramString) {
			super(paramString);
		}

		public UnsupportedImageFormatException(String paramString, Throwable paramThrowable) {
			super(paramString, paramThrowable);
		}

		public UnsupportedImageFormatException(Throwable paramThrowable) {
			super(paramThrowable);
		}
	}
}
