package com.mindoo.dde.imageproperties.internal;

import java.util.logging.Logger;

import org.eclipse.core.runtime.Platform;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

/**
 * The activator class controls the plug-in life cycle
 */
public class Activator extends AbstractUIPlugin {

	// The plug-in ID
	public static final String PLUGIN_ID = "com.mindoo.dde.imageproperties";

	// The shared instance
	private static Activator plugin;
	
	/**
	 * The constructor
	 */
	public Activator() {
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
		plugin = this;
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext context) throws Exception {
		plugin = null;
		super.stop(context);
	}

	/**
	 * Returns the shared instance
	 *
	 * @return the shared instance
	 */
	public static Activator getDefault() {
		return plugin;
	}

	/**
	 * Returns a phrase in the current language from the respective properties-file.<br>
	 * Before the looked up phrase is returned, this method
	 * will replace placeholders like {0}, {1},... {n} within the phrase
	 * with supplied parameters, in the order they are given.<br>
	 * If the phrase is not found, the provided ID is returned.
	 * 
	 * @param phraseID		the id of the phrase
	 * @param parameters 	optional list of parameters to replace placeholders
	 * @return				the phrase
	 */
	public String getPhrase(String phraseID, String... parameters) {
		if (plugin!=null) {
			String phrase=Platform.getResourceString(plugin.getBundle(), "%" + phraseID);
			
			if (phrase==null)
				return phraseID;
			
			for (int i=0; i<parameters.length; i++) {
				String param=parameters[i];
				if (param!=null) {
					phrase=phrase.replace("{"+i+"}",param);
				}
				else
					phrase=phrase.replace("{"+i+"}","null");
			}
			
			return phrase;
		}
		return phraseID;
	}
	
	/**
	 * Returns the logger to be used within this plugin.
	 * 
	 * @return		the logger
	 */
	public static Logger getLogger() {
		return Logger.getLogger(PLUGIN_ID);
	}
}
